/*
 * $RCSfile: BlowfishEncoder.cc,v $
 * $Revision: 1.1 $
 *
 * Comments:
 *
 * (C) Copyright ParaSoft Corporation 1996.  All rights reserved.
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft
 * The copyright notice above does not evidence any
 * actual or intended publication of such source code.
 *
 * $Author: mali $          $Locker:  $
 * $Date: 2008-04-11 12:47:20 $
 * $Log: not supported by cvs2svn $
 * Revision 1.1  2005/07/07 13:43:17  dario
 * added
 *
 */

#include "BlowfishEncoder.h"

static unsigned int Subkeys[] = {
    #include "logfile.dat"
};

BlowfishEncoder::BlowfishEncoder(const char *key, short keybytes)
{
    initialize(key, keybytes);
}

BlowfishEncoder::~BlowfishEncoder()
{
}

void BlowfishEncoder::initialize(const char *key, short keybytes)
{
    short          i;
    short          j;
    short          k;
    short          index = 0;
    unsigned int  data;
    unsigned int  datal;
    unsigned int  datar;
    
    for (i = 0; i < N + 2; ++i) {
        data = Subkeys[index++];
        if (isLittleEndian()) {
            data = ((data & 0xFF000000L) >> 24) |
                ((data & 0x00FF0000L) >>  8) |
                ((data & 0x0000FF00L) <<  8) |
                ((data & 0x000000FFL) << 24);
        }
        P[i] = data;
    }
    
    for (i = 0; i < 4; ++i) {
        for (j = 0; j < 256; ++j) {
            data = Subkeys[index++];
            if (isLittleEndian()) {     
                data = ((data & 0xFF000000L) >> 24) |
                    ((data & 0x00FF0000L) >>  8) |
                    ((data & 0x0000FF00L) <<  8) |
                    ((data & 0x000000FFL) << 24);
            }   
            S[i][j] = data;
        }
    }

    j = 0;
    for (i = 0; i < N + 2; ++i) {
        data = 0x00000000;
        for (k = 0; k < 4; ++k) {
            data = (data << 8) | key[j];
            j = j + 1;
            if (j >= keybytes) {
                j = 0;
            }
        }
        P[i] = P[i] ^ data;
    }
    
    datal = 0x00000000;
    datar = 0x00000000;
    
    for (i = 0; i < N + 2; i += 2) {
        this->write(&datal, &datar);
        
        P[i] = datal;
        P[i + 1] = datar;
    }
    
    for (i = 0; i < 4; ++i) {
        for (j = 0; j < 256; j += 2) {
            
            this->write(&datal, &datar);
            
            S[i][j] = datal;
            S[i][j + 1] = datar;
        }
    }    
    return;
}

unsigned int BlowfishEncoder::F(unsigned int x)
{
    unsigned short a;
    unsigned short b;
    unsigned short c;
    unsigned short d;
    unsigned int  y;
    
    d = x & 0x00FF;
    x >>= 8;
    c = x & 0x00FF;
    x >>= 8;
    b = x & 0x00FF;
    x >>= 8;
    a = x & 0x00FF;
    /*y = ((S[0][a] + S[1][b]) ^ S[2][c]) + S[3][d];*/
    y = S[0][a] + S[1][b];
    y = y ^ S[2][c];
    y = y + S[3][d];
    
    return y;
}

/*
 * blowfish_encipher
 */
void BlowfishEncoder::write(unsigned int *xl, unsigned int *xr)
{
    unsigned int  Xl;
    unsigned int  Xr;
    unsigned int  temp;
    short          i;
    
    Xl = *xl;
    Xr = *xr;
    
    for (i = 0; i < N; ++i) {
        Xl = Xl ^ P[i];
        Xr = F(Xl) ^ Xr;
        
        temp = Xl;
        Xl = Xr;
        Xr = temp;
    }
    
    temp = Xl;
    Xl = Xr;
    Xr = temp;
    
    Xr = Xr ^ P[N];
    Xl = Xl ^ P[N + 1];
    
    *xl = Xl;
    *xr = Xr;
}

void BlowfishEncoder::read(unsigned int *xl, unsigned int *xr)
{
    unsigned int  Xl;
    unsigned int  Xr;
    unsigned int  temp;
    short          i;
    
    Xl = *xl;
    Xr = *xr;
    
    for (i = N + 1; i > 1; --i) {
        Xl = Xl ^ P[i];
        Xr = F(Xl) ^ Xr;
        
        /* Exchange Xl and Xr */
        temp = Xl;
        Xl = Xr;
        Xr = temp;
    }
    
    /* Exchange Xl and Xr */
    temp = Xl;
    Xl = Xr;
    Xr = temp;
    
    Xr = Xr ^ P[1];
    Xl = Xl ^ P[0];
    
    *xl = Xl;
    *xr = Xr;

    return;
}



bool BlowfishEncoder::isLittleEndian()
{    
    char SwapTest[2] = { 1, 0 };
    if( *(short *) SwapTest == 1 ) {
        return true;
    } else {
        return false;
    }
}

