<?xml version="1.0"?>
<!-- in case of any change in xsl/xml file modify all connected xsl and xml files from this package  -->
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" xmlns:graph="graph" xmlns:results="results" extension-element-prefixes="saxon">

    <!-- ================ print API Coverage menu ======================= -->
    <xsl:template name="display_apicoverage_menu">
        <xsl:if test="($apicoverage) > 0">
            <xsl:call-template name="display_linked_menu_item">
                <xsl:with-param name="text" select="$COV_API_COVERAGE" />
                <xsl:with-param name="link">#apicov</xsl:with-param>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <!-- =================  print API Coverage header ============================ -->
    <xsl:template name="apicoverage_header">
        <a name="apicov" />
        <table width="{$width}" border="0" bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
            <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">15% 70% 15%</xsl:attribute>
            </xsl:if>
            <tr>
                <td width="15%"></td>
                <td align="center">
                    <div class="header"><xsl:value-of select="$COV_API_COVERAGE" /></div>
                </td>
                <td width="15%" align="right" valign="bottom">
                    <xsl:if test="not($report_type = $email_report) and ($first_back_to_top = 'false')">
                        <xsl:call-template name="back_to_top" />
                    </xsl:if>
                    <saxon:assign name="first_back_to_top">false</saxon:assign>
                </td>
            </tr>
        </table>
        <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
            <div style="padding-left: 5px; padding-bottom: 2px;">
                <a class="blue_link" style="margin-right: 5px;" href="#" onclick="expandAll()">[Expand All]</a>
                <a class="blue_link" href="#" onclick="collapseAll()">[Collapse All]</a>
            </div>
        </xsl:if>
    </xsl:template>

    <!-- ==================  print API Coverage ============================ -->
    <xsl:template name="display_apicoverage">
        <xsl:if test="($apicoverage > 0)">
            <xsl:call-template name="apicoverage_header" />
            <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                <style>
                    .arrow {
                        cursor: pointer;
                        color: #666;
                    }
                    @media print {
                        .bar_filler {
                            width:100%;
                            height:0px;
                            border-bottom:14px solid #008000;
                        }
                    }
                    .detailLink {
                        text-decoration: underline;
                        font-family: Arial, Helvetica, sans-serif;
                        font-style: normal;
                        color: #990000;
                        padding-left: 2em;
                        cursor: pointer;
                    }
                </style>
                <style type="text/css">
                    TD.testdetails{ border-top: 1px solid white; border-bottom: 1px solid white; border: 1px solid white; background-color: #f3f3f3}
                </style>
                <script>
                    if (document.documentMode &amp;&amp; document.documentMode == 5) {
                        var css = ".bar_filler {margin-top: -15px;}",
                        head = document.head || document.getElementsByTagName('head')[0],
                        style = document.createElement('style');
                        style.type = 'text/css';
                        if (style.styleSheet){
                            style.styleSheet.cssText = css;
                        } else {
                            style.appendChild(document.createTextNode(css));
                        }
                        head.appendChild(style);
                    }
                </script>
            </xsl:if>
            <table id="apicoveragetbl" class="singleborder" width='{$width}' border='0' bordercolor='{$func_bdrcolor}' cellpadding='2' cellspacing='0'>
                <xsl:if test="$report_format_mode='pdf'">
                    <xsl:attribute name="cols">60% 10% 30%</xsl:attribute>
                </xsl:if>
                <tr>
                    <td align="center" class="singleborder" bgcolor="{$func_bcolor}" nowrap="nowrap" width="60%"><b><xsl:value-of select="$COV_COVERABLE_RESOURCE" /></b></td>
                    <td align="center" class="singleborder" bgcolor="{$func_bcolor}" nowrap="nowrap" width="10%"><b><xsl:value-of select="$COV_SUCCESS_TOTAL" /></b></td>
                    <td align="center" class="singleborder" bgcolor="{$func_bcolor}" nowrap="nowrap" width="30%"><b><xsl:value-of select="$COV_COVERAGE_LOWER" /></b></td>
                </tr>
                <xsl:apply-templates select="/ResultsSession/APICoverage/service" />
            </table>
            <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                <script language="JavaScript">
                    // This script handles show/hide of table rows in the API Coverage section.
                    // We use an array of booleans to track whether each row (node) in the table
                    // has been collapsed or not.
                    var collapse = [];
                    var details = [];
                    function init() {
                        // Builds the array of booleans - only called once after the table loads
                        var tbl = document.getElementById("apicoveragetbl");
                        for (var i = 1; i &lt; tbl.rows.length; i++) {
                            // collapse all rows by default
                            var row = tbl.rows[i];
                            var level = parseInt(row.getAttribute("level"));
                            collapse[i] = true;
                            if (isTestDetails(row)) {
                                details[i] = false;
                            }
                            updateRow(row, (level != 1));
                            updateArrow(row, true);
                        }
                    }
                    init();

                    function isTestDetails(row) {
                        return !!row.getAttribute("test_detail");
                    }

                    function clickRow(idx) {
                        // Called when a row is clicked - needs to show/hide child nodes
                        var tbl = document.getElementById("apicoveragetbl");
                        var row = tbl.rows[idx];
                        var level = parseInt(row.getAttribute("level"));
                        collapse[idx] = !collapse[idx];
                        updateArrow(row, collapse[idx]);
                        for (var i = idx+1; i &lt; tbl.rows.length; i++) {
                            // We go through each row, starting at the row after the one that
                            // was clicked (first potential child) to the next node at the same
                            // level (which cannot be a child)
                            var cur_level = parseInt(tbl.rows[i].getAttribute("level"));
                            if (cur_level &gt; level) {
                                // This is a child node - update it
                                // Note: we never hide root nodes
                                var currentRow = tbl.rows[i];
                                if (!isTestDetails(currentRow)) {
                                    updateRow(currentRow, shouldHide(tbl, i));
                                }
                            } else {
                                // Node at same or higher level - cannot be a child - quit here
                                break;
                            }
                        }
                    }

                    function updateArrow(row, collapsed) {
                        var spans = row.cells[0].getElementsByTagName("span");
                        if (spans.length &gt; 0) {
                            var link = spans[0].getAttribute("isDetailLink");
                            if (!link) {
                                spans[0].innerHTML = collapsed ? "&#9658;" : "&#9660;";
                            }
                        }
                    }

                    function shouldHide(tbl, idx) {
                        // Searches for a parent node for which collapse == true
                        var level = tbl.rows[idx].getAttribute("level");
                        for (var i = idx-1; i &gt; 0; i--) {
                            var cur_level = parseInt(tbl.rows[i].getAttribute("level"));
                            if (cur_level &lt; level) {
                                if (collapse[i]) {
                                    return true;
                                } else if (cur_level == 1) {
                                    // Stop at top level - we don't want to look outside our tree
                                    return false;
                                }
                                level = cur_level;
                            }
                        }
                        return false;
                    }

                    function updateRow(row, hide) {
                        // Either hides or shows a given row
                        row.style.display = hide ? 'none' : 'table-row';
                        var spans = row.cells[0].getElementsByTagName("span");
                        if (spans.length &gt; 0) {
                            var link = spans[0].getAttribute("isDetailLink");
                            if (link &amp;&amp; link == "true") {
                                if (details[row.rowIndex] &amp;&amp; hide) {
                                    toggleDetails(row.cells[0]);
                                    hideDetailLink(row.cells[0]);
                                }
                            }
                        }
                    }

                    function showDetailLink(tableData) {
                        var tbl = document.getElementById("apicoveragetbl");
                        var rowId = tableData.parentNode.rowIndex;
                        var detailsRow = tbl.rows[rowId + 1];
                        if (detailsRow) {
                            var hasDetail = detailsRow.getAttribute("test_detail");
                            if (hasDetail) {
                                var spans = tableData.getElementsByTagName("span");
                                if (spans.length &gt; 0) {
                                    spans[0].style.display = "inline";
                                }
                            }
                        }
                    }

                    function hideDetailLink(tableData) {
                        var spans = tableData.getElementsByTagName("span");
                        var rowId = tableData.parentNode.rowIndex;
                        if (!details[rowId] &amp;&amp; spans.length &gt; 0) {
                            spans[0].style.display = "none";
                        }
                    }

                    function toggleScopeDetails(event, tableData) {
                        //prevent bubbling
                        event = event || window.event;
                        if (event.stopPropagation) {
                            event.stopPropagation();
                        } else {
                            event.cancelBubble = true;
                        }
                        toggleDetails(tableData);
                    }

                    function toggleDetails(tableData) {
                        var tbl = document.getElementById("apicoveragetbl");
                        var rowId = tableData.parentNode.rowIndex;
                        var spans = tableData.getElementsByTagName("span");
                        details[rowId] = !details[rowId];
                        if (spans.length &gt; 0) {
                            if (details[rowId]) {
                                spans[0].innerHTML = "Hide tests";
                            } else {
                                spans[0].innerHTML = "Show tests";
                            }
                        }
                        for (var i = rowId+1; i &lt; tbl.rows.length; i++) {
                            row = tbl.rows[i];
                            var testDetail = row.getAttribute("test_detail");
                            if (testDetail) {
                                var display = row.style.display;
                                if (display == "none") {
                                    row.style.display="table-row";
                                } else {
                                    row.style.display="none";
                                }
                            } else {
                                break;
                            }
                        }
                    }

                    function expandAll() {
                        var tbl = document.getElementById("apicoveragetbl");
                        for (var i = 1; i &lt; tbl.rows.length; i++) {
                            var row = tbl.rows[i];
                            collapse[i] = false;
                            updateArrow(row, false);
                            if (!isTestDetails(row)) {
                                updateRow(row, false);
                            }
                        }
                    }

                    function collapseAll() {
                        var tbl = document.getElementById("apicoveragetbl");
                        for (var i = 1; i &lt; tbl.rows.length; i++) {
                            var row = tbl.rows[i];
                            var level = parseInt(row.getAttribute("level"));
                            collapse[i] = true;
                            updateArrow(row, true);
                            if (level > 1) {
                                updateRow(row, true);
                            }
                        }
                    }
                </script>
            </xsl:if>
        </xsl:if>
    </xsl:template>

    <xsl:template name="apicoverage_detail" match="service">
        <tr>
            <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                <xsl:attribute name="onclick">clickRow(this.rowIndex);</xsl:attribute>
                <xsl:attribute name="level">1</xsl:attribute>
            </xsl:if>
            <xsl:variable name="id">
                <xsl:choose>
                    <xsl:when test="@definition-uri"><xsl:value-of select="@definition-uri" /></xsl:when>
                    <xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <td class="singleborder">
                <xsl:choose>
                    <xsl:when test="$report_format_mode='pdf' or $report_type = $email_report">
                        <xsl:call-template name="split_if_needed"><xsl:with-param name="value" select="$id"/></xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <div style="word-wrap: break-word; max-width: 680px;">
                            <span class="arrow">&#9660;</span><xsl:value-of select="$id"/>
                        </div>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <xsl:call-template name="coverage_counts" />
        </tr>
        <xsl:apply-templates select="resource" />
    </xsl:template>

    <xsl:template match="resource">
        <tr>
            <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                <xsl:attribute name="onclick">clickRow(this.rowIndex);</xsl:attribute>
                <xsl:attribute name="level">2</xsl:attribute>
            </xsl:if>
            <td class="singleborder">
                <xsl:choose>
                    <xsl:when test="$report_format_mode='pdf' or $report_type = $email_report">
                        <xsl:call-template name="repeat_n"><xsl:with-param name="str" select="'&#160;'"/><xsl:with-param name="n" select="4"/></xsl:call-template>
                        <xsl:call-template name="split_if_needed">
                            <xsl:with-param name="value" select="@id" />
                            <xsl:with-param name="indent" select="1"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <div style="padding-left: 2em; word-wrap: break-word; max-width: 660px;">
                            <span class="arrow">&#9660;</span><xsl:value-of select="@id"/>
                        </div>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <xsl:call-template name="coverage_counts" />
        </tr>
        <xsl:apply-templates select="operation" />
    </xsl:template>

    <xsl:template match="operation">
        <tr>
            <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                <xsl:attribute name="onclick">clickRow(this.rowIndex);</xsl:attribute>
                <xsl:attribute name="level">3</xsl:attribute>
            </xsl:if>
            <td class="singleborder" onMouseOut="hideDetailLink(this)" onMouseOver="showDetailLink(this)">
                <xsl:choose>
                    <xsl:when test="$report_format_mode='pdf' or $report_type = $email_report">
                        <xsl:call-template name="repeat_n"><xsl:with-param name="str" select="'&#160;'"/><xsl:with-param name="n" select="8"/></xsl:call-template>
                        <xsl:call-template name="split_if_needed">
                            <xsl:with-param name="value" select="@name" />
                            <xsl:with-param name="indent" select="2"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <div style="padding-left: 4em; word-wrap: break-word; max-width: 580px; display: inline;">
                            <xsl:value-of select="@name"/>
                        </div>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                    <span style="display:none;" isDetailLink="true" class="detailLink" onclick="toggleScopeDetails(event, this.parentNode);">Show tests</span>
                </xsl:if>
            </td>
            <xsl:call-template name="coverage_counts" />
        </tr>
        <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
            <xsl:apply-templates select="test-ref[generate-id() = generate-id(key('tstFile', concat(ancestor::service/@definition-uri, ancestor::resource/@id, ancestor::operation/@name, substring-before(@id,'#')))[1])]" />
        </xsl:if>
    </xsl:template>

    <xsl:key name="tstFile" match="test-ref" use="concat(ancestor::service/@definition-uri, ancestor::resource/@id, ancestor::operation/@name, substring-before(@id,'#'))" />

    <xsl:template match="test-ref">
        <xsl:variable name="id" select="@id" />
        <!-- create tests node set for tests identifiers  -->
        <xsl:variable name="testIds" select="..//test-ref[substring-before(@id,'#') = substring-before(($id),'#')]" />
        <xsl:variable name="testNodes" select="/ResultsSession/ExecutedTestsDetails//Test[@id = $testIds/@id]" />
        <tr class="detailRow">
            <xsl:attribute name="level">4</xsl:attribute>
            <xsl:attribute name="test_detail">true</xsl:attribute>
            <td class="testdetails">
                <div style="padding-left: 6em; word-wrap: break-word; max-width: 620px;">
                    <xsl:call-template name="get_name_from_id">
                        <xsl:with-param name="id" select="@id" />
                    </xsl:call-template>
                </div>
            </td>

            <xsl:call-template name="apicoverage_columns">
                <xsl:with-param name="passed"><xsl:call-template name="get_attr_sum_by_nodes"><xsl:with-param name="nodes" select="$testNodes" /><xsl:with-param name="attr">pass</xsl:with-param></xsl:call-template></xsl:with-param>
                <xsl:with-param name="failed"><xsl:call-template name="get_attr_sum_by_nodes"><xsl:with-param name="nodes" select="$testNodes" /><xsl:with-param name="attr">fail</xsl:with-param></xsl:call-template></xsl:with-param>
                <xsl:with-param name="tested" select="1" />
                <xsl:with-param name="total" select="1" />
                <xsl:with-param name="isUnknown"><xsl:choose><xsl:when test="@definition-uri or ancestor::*/@definition-uri">false</xsl:when><xsl:otherwise>true</xsl:otherwise></xsl:choose></xsl:with-param>
                <xsl:with-param name="isDetails">true</xsl:with-param>
            </xsl:call-template>
        </tr>
        <xsl:if test="$test_suites_only != 'true'">
            <xsl:call-template name="test_cases_rows">
                <xsl:with-param name="nodes" select="$testIds" />
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template name="test_cases_rows">
        <xsl:param name="nodes"></xsl:param>
        <xsl:for-each select="$nodes">
            <xsl:variable name="id" select="@id" />
            <!-- find test node corresponding to id -->
            <xsl:variable name="testNode" select="/ResultsSession/ExecutedTestsDetails//Test[@id = $id]" />
            <tr class="detailRow">
                <xsl:attribute name="level">5</xsl:attribute>
                <xsl:attribute name="test_detail">true</xsl:attribute>
                <td class="testdetails">
                    <div style="padding-left: 8em; word-wrap: break-word; max-width: 600px;">
                        <xsl:call-template name="get_test_path"><xsl:with-param name="node" select="$testNode" /></xsl:call-template>
                    </div>
                </td>
                <xsl:call-template name="apicoverage_columns">
                    <xsl:with-param name="passed"><xsl:call-template name="get_attr_sum_by_nodes"><xsl:with-param name="nodes" select="$testNode" /><xsl:with-param name="attr">pass</xsl:with-param></xsl:call-template></xsl:with-param>
                    <xsl:with-param name="failed"><xsl:call-template name="get_attr_sum_by_nodes"><xsl:with-param name="nodes" select="$testNode" /><xsl:with-param name="attr">fail</xsl:with-param></xsl:call-template></xsl:with-param>
                    <xsl:with-param name="tested" select="1" />
                    <xsl:with-param name="total" select="1" />
                    <xsl:with-param name="isUnknown"><xsl:choose><xsl:when test="@definition-uri or ancestor::*/@definition-uri">false</xsl:when><xsl:otherwise>true</xsl:otherwise></xsl:choose></xsl:with-param>
                    <xsl:with-param name="isDetails">true</xsl:with-param>
                </xsl:call-template>
            </tr>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="apicoverage_columns">
        <xsl:param name="passed" />
        <xsl:param name="failed" />
        <xsl:param name="tested" />
        <xsl:param name="total" />
        <xsl:param name="isUnknown" />
        <xsl:param name="isDetails" />
        <td class="singleborder" nowrap="nowrap" align="center">
            <xsl:if test="$isDetails = 'true'">
                <xsl:attribute name="class">testdetails</xsl:attribute>
            </xsl:if>
            <xsl:value-of select="$passed" />&#160;/&#160;<xsl:value-of select="$passed + $failed" />
        </td>
        <td class="singleborder" align="center">
            <xsl:if test="$isDetails = 'true'">
                <xsl:attribute name="class">testdetails</xsl:attribute>
            </xsl:if>
            <xsl:choose>
                <xsl:when test="$isUnknown = 'true'">&#160;</xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="passedPercent">
                        <xsl:choose>
                            <xsl:when test="$total = 0">0%</xsl:when>
                            <xsl:otherwise><xsl:value-of select="format-number(($tested div $total) * ($passed div ($passed + $failed)), '#%')" /></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="failedPercent">
                        <xsl:choose>
                            <xsl:when test="$total = 0">0%</xsl:when>
                            <xsl:otherwise><xsl:value-of select="format-number(($tested div $total) * ($failed div ($passed + $failed)), '#%')" /></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="notTestedPercent">
                        <xsl:choose>
                            <xsl:when test="$total = 0">100%</xsl:when>
                            <xsl:otherwise><xsl:value-of select="format-number(1 - ($tested div $total), '#%')" /></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <table cellpadding="0" cellspacing="0" border="0" margin="0pt" width="300">
                        <xsl:if test="$report_format_mode='pdf'">
                            <xsl:attribute name="cols">17% 23% 60%</xsl:attribute>
                        </xsl:if>
                        <tr>
                            <td nowrap="nowrap" align="right" width="50" style="padding-right: 10px;">
                                <xsl:value-of select="$tested" />&#160;/&#160;<xsl:value-of select="$total" />
                            </td>
                            <td nowrap="nowrap" width="50">
                                <xsl:if test="$report_format_mode = 'pdf'">&#160;&#160;</xsl:if>(<xsl:value-of select="format-number($tested div $total, '#%')" />)
                            </td>
                            <td nowrap="nowrap" style="padding: 2px; padding-left: 10px;" width="200">
                                <table width="200" height="18" cellpadding="0" cellspacing="0" bordercolor="{$func_bdrcolor}" bgcolor="{$func_bcolor}" margin="0pt" style="background-color: {$func_bcolor}; border: {$func_bdrcolor} 1px solid; width: 200px; height:18px; padding:0;" border="0">
                                    <xsl:if test="$report_format_mode='pdf'">
                                        <xsl:attribute name="cols">
                                            <xsl:if test="$passed > 0"><xsl:value-of select="$passedPercent" /></xsl:if>
                                            <xsl:if test="$tested > 0 and $failed > 0"><xsl:text> </xsl:text></xsl:if>
                                            <xsl:if test="$failed > 0"><xsl:value-of select="$failedPercent" /></xsl:if>
                                            <xsl:if test="($failed > 0 or $passed > 0) and $tested != $total"><xsl:text> </xsl:text></xsl:if>
                                            <xsl:if test="$tested != $total"><xsl:value-of select="$notTestedPercent" /></xsl:if>
                                        </xsl:attribute>
                                    </xsl:if>
                                    <tr>
                                        <xsl:if test="$passed > 0">
                                            <td bgcolor="#5cb85c">
                                                <xsl:attribute name="style">border: <xsl:value-of select="$func_bcolor"/> solid 1px; background-color: #008000; font-size: xx-small;<xsl:if test="$failed > 0"> border-right: none;</xsl:if></xsl:attribute>
                                                <xsl:attribute name="width"><xsl:value-of select="$passedPercent" /></xsl:attribute>
                                                <div class="bar_filler"><xsl:if test="($report_format_mode = 'pdf') or ($report_type = $email_report)">&#160;</xsl:if></div>
                                            </td>
                                        </xsl:if>
                                        <xsl:if test="$failed > 0">
                                            <td bgcolor="#d9534f">
                                                <xsl:attribute name="style">border: <xsl:value-of select="$func_bcolor"/> solid 1px; background-color: #80000; font-size: xx-small;<xsl:if test="$passed > 0"> border-left: none;</xsl:if></xsl:attribute>
                                                <xsl:attribute name="width"><xsl:value-of select="$failedPercent" /></xsl:attribute>
                                                <div class="bar_filler"><xsl:if test="($report_format_mode = 'pdf') or ($report_type = $email_report)">&#160;</xsl:if></div>
                                            </td>
                                        </xsl:if>
                                        <xsl:if test="$tested != $total">
                                            <td bgcolor="{$func_bcolor}" style="border: {$func_bcolor} solid 1px; background-color: {$func_bcolor};">
                                                <xsl:attribute name="width"><xsl:value-of select="$notTestedPercent" /></xsl:attribute>
                                                <xsl:if test="($report_format_mode = 'pdf') or ($report_type = $email_report)">&#160;</xsl:if>
                                            </td>
                                        </xsl:if>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                    </table>
                </xsl:otherwise>
            </xsl:choose>
        </td>
    </xsl:template>

    <xsl:template name="coverage_counts">
        <xsl:variable name="tested" select="count(.//test-ref/..)" />
        <xsl:variable name="untested">
            <xsl:choose>
                <xsl:when test="*"><xsl:value-of select="count(.//*[not(*) and not(self::test-ref)])" /></xsl:when>
                <xsl:otherwise>1</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <!-- create tests node set for tests identifiers  -->
        <xsl:variable name="testIds" select=".//test-ref" />
        <xsl:variable name="testNodes" select="/ResultsSession/ExecutedTestsDetails//Test[@id = $testIds/@id]" />

        <xsl:call-template name="apicoverage_columns">
            <xsl:with-param name="passed"><xsl:call-template name="get_attr_sum_by_nodes"><xsl:with-param name="nodes" select="$testNodes" /><xsl:with-param name="attr">pass</xsl:with-param></xsl:call-template></xsl:with-param>
            <xsl:with-param name="failed"><xsl:call-template name="get_attr_sum_by_nodes"><xsl:with-param name="nodes" select="$testNodes" /><xsl:with-param name="attr">fail</xsl:with-param></xsl:call-template></xsl:with-param>
            <xsl:with-param name="tested" select="$tested" />
            <xsl:with-param name="total" select="$tested + $untested" />
            <xsl:with-param name="isUnknown"><xsl:choose><xsl:when test="@definition-uri or ancestor::*/@definition-uri">false</xsl:when><xsl:otherwise>true</xsl:otherwise></xsl:choose></xsl:with-param>
            <xsl:with-param name="isDetails">false</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <!-- Gets a sum of attribute values from Test nodes.
         Useful in calculating pass/fail numbers -->
    <xsl:template name="get_attr_sum_by_nodes">
        <xsl:param name="nodes"></xsl:param>
        <xsl:param name="attr" />
        <xsl:variable name="ret" saxon:assignable="yes" select="0" />
        <xsl:for-each select="$nodes">
            <xsl:variable name="val" select="./@*[local-name()=$attr]" />
            <xsl:if test="$val">
                <saxon:assign name="ret" select="$ret + $val" />
            </xsl:if>
        </xsl:for-each>
        <xsl:value-of select="$ret" />
    </xsl:template>

    <xsl:template name="get_name_from_id">
        <xsl:param name="id" />
        <xsl:variable name="trim_start" select="substring-after(($id),':///')" />
        <xsl:variable name="clean_path" select="substring-before(($trim_start),'#')" saxon:assignable="yes" />
        <xsl:if test="not(string-length($clean_path) > 0)">
            <saxon:assign name="clean_path" select="substring-before(($trim_start),'.')" />
        </xsl:if>
        <xsl:value-of select="($clean_path)" />
    </xsl:template>

    <!-- Gets the path of Test Suites within the .tst file to the specified Test node -->
    <xsl:template name="get_test_path">
        <xsl:param name="node"></xsl:param>
        <xsl:for-each select="$node">
            <xsl:for-each select="ancestor-or-self::*[local-name()='TestSuite' or local-name()='Test'][contains(@id, '#')]">
                <xsl:if test="position() > 1"><xsl:text>/</xsl:text></xsl:if>
                <xsl:value-of select="@name" />
            </xsl:for-each>
        </xsl:for-each>
    </xsl:template>
</xsl:stylesheet>
