<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" extension-element-prefixes="saxon">

	<xsl:output method="xml" encoding="UTF-8" indent="yes" />
	<xsl:template match="/">
		<xsl:choose>
			<xsl:when test="ResultsSession/@toolName!='SOAtest'">
				<xsl:apply-templates select="ResultsSession/ExecutedTestsDetails"></xsl:apply-templates>				
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="ExecutedTestsDetails">
		<xsl:apply-templates select="Total" />
	</xsl:template>
	
	<xsl:template match="TestConfig">
		<config_found/>		
	</xsl:template>
	
	<xsl:template match="Authors">
		<auth_found/>		
	</xsl:template>
	
	<xsl:template match="Goals">
		<goals_found/>		
	</xsl:template>
	
	<xsl:template match="VersionInfos">
		<ver_found/>		
	</xsl:template>

	<xsl:template match="Total">    
		<testsuites tests="{@total}" failures="{@fail}" errors="{@err}">
      <xsl:call-template name="addTimeAttribute" />      
			<xsl:apply-templates select="Project" />
		</testsuites>
	</xsl:template>

                         
	<xsl:template match="Project//TestSuite[count(child::TestSuite[count(child::Test) > 0]) > 0]">    
    	<xsl:variable name="suitename" saxon:assignable="yes" select="ancestor-or-self::Project/@name" />
    	<xsl:for-each select="ancestor-or-self::TestSuite">      
		    <saxon:assign name="suitename" select="concat($suitename,'/',@name)"/>
	  	</xsl:for-each>    
		<testsuite name="{$suitename}" tests="{count(.//Test)}" failures="{@fail}" errors="{@err}">      
         	<xsl:call-template name="addTimeAttribute" />      
		 	<xsl:apply-templates select=".//Test" />
		</testsuite>
	</xsl:template>
	
	<xsl:template match="Project/TestSuite[count(child::Test) > 0]">    
    	<xsl:variable name="suitename" select="../@name" />    	   
		<testsuite name="{$suitename}" tests="{count(.//Test)}" failures="{@fail}" errors="{@err}">      
         	<xsl:call-template name="addTimeAttribute" />      
		 	<xsl:apply-templates select=".//Test" />
		</testsuite>
	</xsl:template>
			
  <xsl:template name="addTimeAttribute">    
    <xsl:attribute name="time">
      <xsl:call-template name="timeFormat">
		<xsl:with-param name="initTime" select="@time" />
      </xsl:call-template>      
    </xsl:attribute>
  </xsl:template>

	<xsl:template match="Test">
	<xsl:variable name="status" saxon:assignable="yes">pass</xsl:variable>	
    <xsl:if test="(@err) > 0">    
    	<saxon:assign name="status">err</saxon:assign>
    </xsl:if>
    <xsl:if test="not((@err) > 0)">
    	<xsl:if test="(@fail) > 0">
    		<saxon:assign name="status">fail</saxon:assign>
    	</xsl:if>
    </xsl:if>
		<xsl:choose>
			<xsl:when test="TestCase">				
					<xsl:for-each select="TestCase">
						<xsl:call-template name="writeXUnitTestCase">
							<xsl:with-param name="id" select="@testId" />
							<xsl:with-param name="time" select="@time" />
							<xsl:with-param name="status" select="$status" />
							<xsl:with-param name="tcId" select="@id" />
						</xsl:call-template>
					</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeXUnitTestCase">
					<xsl:with-param name="id" select="@id" />
					<xsl:with-param name="time" select="@time" />
					<xsl:with-param name="status" select="$status" />
					<xsl:with-param name="tcId" select="'null'" />
				</xsl:call-template>

			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template name="writeXUnitTestCase">
		<xsl:param name="id" />
		<xsl:param name="time" />
		<xsl:param name="status" />
		<xsl:param name="tcId" select="'null'" />

		<xsl:variable name="timeInMillis">
			<xsl:call-template name="timeFormat">
				<xsl:with-param name="initTime" select="$time" />
			</xsl:call-template>
		</xsl:variable>			
		<xsl:choose>
			<xsl:when test="$tcId='null'">
				<xsl:variable name="testname" select="../@name" />		
				<xsl:variable name="name" select="@name" saxon:assignable="yes"/>		
				<xsl:if test="string-length(@name) = 0">
					<saxon:assign name="name">[no name]</saxon:assign>
				</xsl:if>	
				<xsl:if test="$status!='pass'">
					<testcase name="{$name}" time="{$timeInMillis}" classname="{$testname}">					 
						<xsl:for-each select="/ResultsSession/Exec/ExecViols/ExecViol[@testId=$id]">
						<xsl:call-template name="ExecViol">
							<xsl:with-param name="execViol" select="." />
							<xsl:with-param name="status" select="$status" />
						</xsl:call-template>
						</xsl:for-each>
					</testcase>
				</xsl:if>
				<xsl:if test="$status='pass'">
					<testcase name="{$name}" time="{$timeInMillis}" classname="{$testname}">
					</testcase>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="testname" select="../../@name" />		
				<xsl:variable name="name" select="../@name" saxon:assignable="yes"/>		
				<xsl:if test="string-length(../@name) = 0">
					<saxon:assign name="name">[no name]</saxon:assign>
				</xsl:if>
				<xsl:if test="$status!='pass'">					
					<testcase name="{$name}" time="{$timeInMillis}" classname="{$testname}">
						<xsl:if test="@fail > 0 or @err > 0">					 
							<xsl:for-each select="/ResultsSession/Exec/ExecViols/ExecViol[@testId=$id and @testCaseId=$tcId]">
								<xsl:call-template name="ExecViol">
									<xsl:with-param name="execViol" select="." />
									<xsl:with-param name="status" select="$status" />
								</xsl:call-template>
							</xsl:for-each>
						</xsl:if>						
					</testcase>
				</xsl:if>
				<xsl:if test="$status='pass'">
					<testcase name="{$name}" time="{$timeInMillis}" classname="{$testname}">
					</testcase>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="timeFormat">
		<xsl:param name="initTime" />
		
		<xsl:if test="string-length($initTime) > 0">
		    <xsl:variable name="millis">
				<xsl:value-of select="substring($initTime,9,3)" />
			</xsl:variable>
	
			<xsl:variable name="sec">
				<xsl:value-of select="substring($initTime,6,2)" />
			</xsl:variable>
	
			<xsl:variable name="min">
				<xsl:value-of select="substring($initTime,3,2)" />
			</xsl:variable>
	
			<xsl:variable name="hour">
				<xsl:value-of select="substring($initTime,1,1)" />
			</xsl:variable>
	
			<xsl:variable name="allSec"> 
				<xsl:value-of select="$sec + (60 * $min) + (3600 * $hour)" />
			</xsl:variable>
	
			<xsl:value-of select="concat($allSec,'.',$millis)" />         
	    </xsl:if>  
	    <xsl:if test="not(string-length($initTime) > 0)">0.000</xsl:if>			
	</xsl:template>

	<xsl:template name="ExecViol">
		<xsl:param name="execViol" />
		<xsl:param name="status" />
		<xsl:choose>
			<xsl:when test="$status='err'">
				<error>
					<xsl:value-of select="$execViol/Thr/ThrPart/@prnMsg"></xsl:value-of>
				</error>
			</xsl:when>
			<xsl:when test="$status='fail'">
				<failure>
					<xsl:value-of select="$execViol/Thr/ThrPart/@prnMsg"></xsl:value-of>
				</failure>
			</xsl:when>
		</xsl:choose>
	</xsl:template>



</xsl:stylesheet>
